import { createEditor, PluginParams } from '@modern-js-block/runtime';
import {
  getShapeStyle,
  mapToOptions,
  rendererMap,
  getAxisControls,
  getDataLabelControls,
  getLegendControls,
  getTooltipControls,
} from './shared';
import LineChart from '@/LineChart';

createEditor(LineChart, (params: PluginParams) => {
  const { ControlType, addPropertyControls, props } = params;
  addPropertyControls({
    data: {
      type: ControlType.Json,
      title: '图表数据',
    },
    metaGroup: {
      type: ControlType.Group,
      title: '画布',
      propertyControls: {
        title: {
          type: ControlType.String,
          title: '标题',
          defaultValue: '多折线图',
        },
        subTitle: {
          type: ControlType.String,
          title: '副标题',
          defaultValue: '一个简单的折线图',
        },
      },
    },
    xAxisGroup: {
      type: ControlType.Group,
      title: 'X 轴',
      propertyControls: {
        xField: {
          type: ControlType.Enum,
          title: '属性名',
          options: (props.data?.length &&
            Object.keys(props.data[0]).map(key => ({
              label: key,
              value: key,
            }))) || ['x'],
          autoComplete: true,
          defaultValue: 'x',
        },
        xAxis: {
          type: ControlType.Object,
          title: '更多配置',
          propertyControls: getAxisControls(params),
        },
      },
    },
    yAxisGroup: {
      type: ControlType.Group,
      title: 'Y 轴',
      propertyControls: {
        yFields: {
          type: ControlType.Array,
          title: '属性名（多选）',
          tips: '你可以选择多个属性作为 y 值，由此得到多条曲线。（若多选，则分组属性名默认失效）',
          propertyControl: {
            type: ControlType.Enum,
            title: '属性名',
            options: (props.data?.length &&
              Object.keys(props.data[0]).map(key => ({
                label: key,
                value: key,
              }))) || ['y1', 'y2'],
            autoComplete: true,
          },
          'x-linkages': [
            {
              type: 'value:visible',
              target: 'seriesField',
              condition: '{{ $self.value?.length < 2 }}',
            },
          ],
          defaultValue: ['y1', 'y2'],
        },
        yAxis: {
          type: ControlType.Object,
          title: '更多配置',
          propertyControls: getAxisControls(params),
          defaultValue: {
            grid: {
              line: {
                style: {
                  stroke: '#F2F3F5',
                },
              },
            },
          },
        },
      },
    },
    seriesField: {
      type: ControlType.Enum,
      title: '分组属性名',
      tips: '选取一个属性用作分类，不同的值对应不同线段，由此得到多条线段。',
      options: (props.data?.length &&
        Object.keys(props.data[0]).map(key => ({
          label: key,
          value: key,
        }))) || ['y'],
      autoComplete: true,
    },
    legendGroup: {
      type: ControlType.Group,
      title: '图例',
      propertyControls: {
        legendPosition: {
          type: ControlType.Enum,
          title: '方位',
          options: mapToOptions({
            top: '顶部',
            left: '左侧',
            right: '右侧',
            bottom: '底侧',
          }),
          defaultValue: 'bottom',
        },
        legendSubPosition: {
          type: ControlType.Enum,
          title: ['top', 'bottom'].includes(props.legendPosition)
            ? '水平位置'
            : '垂直位置',
          options: ['top', 'bottom'].includes(props.legendPosition)
            ? mapToOptions({
                left: '左',
                middle: '中',
                right: '右',
              })
            : mapToOptions({
                top: '上',
                middle: '中',
                bottom: '下',
              }),
          defaultValue: 'left',
        },
      },
    },
    styleGroup: {
      type: ControlType.Group,
      title: '基础样式',
      propertyControls: {
        lineStyleGroup: {
          type: ControlType.Group,
          title: '线段',
          propertyControls: {
            smooth: {
              type: ControlType.Boolean,
              title: '曲线是否平滑',
              defaultValue: false,
            },
            lineSize: {
              type: ControlType.Number,
              title: '线段粗细',
              defaultValue: 2,
              min: 1,
            },
          },
        },
        pointStyleGroup: {
          type: ControlType.Group,
          title: '数据点',
          propertyControls: {
            point: {
              type: ControlType.Object,
              title: '样式',
              propertyControls: {
                color: {
                  type: ControlType.String,
                  title: '数据点颜色',
                },
                shape: {
                  type: ControlType.String,
                  title: '数据点形状',
                },
                size: {
                  type: ControlType.Number,
                  title: '数据点大小',
                },
                style: {
                  type: ControlType.Json,
                  title: '数据点样式',
                },
              },
            },
            label: {
              type: ControlType.Object,
              title: '标签',
              propertyControls: getDataLabelControls(params),
            },
            tooltip: {
              type: ControlType.Object,
              title: 'Tooltip',
              propertyControls: getTooltipControls(params),
              defaultValue: {
                shared: true,
              },
            },
          },
        },
        canvasStyleGroup: {
          type: ControlType.Group,
          title: '画布',
          propertyControls: {
            width: {
              type: ControlType.Number,
              title: '宽度',
              defaultValue: 400,
            },
            height: {
              type: ControlType.Number,
              title: '高度',
              defaultValue: 400,
            },
          },
          defaultOpen: false,
        },
        meta: {
          type: ControlType.Json,
          title: 'Meta 信息',
        },
        lineStyle: {
          type: ControlType.Object,
          title: '折线图形样式',
          propertyControls: getShapeStyle(params),
          defaultValue: {
            lineWidth: 1,
          },
        },
      },
      defaultOpen: false,
    },
    advancedGroup: {
      type: ControlType.Group,
      title: '高级功能',
      propertyControls: {
        legend: {
          type: ControlType.Object,
          title: '图例 Legend',
          propertyControls: getLegendControls(params),
        },
        autoFit: {
          type: ControlType.Boolean,
          title: '自适应',
        },
        renderer: {
          type: ControlType.Enum,
          title: '渲染方式',
          options: mapToOptions(rendererMap),
        },
        limitInPlot: {
          type: ControlType.Boolean,
          title: '超出内容裁剪',
        },
        isStack: {
          tips: '对于存在 seriesField 分组属性的情况，我们可以设置 isStack = true，让折线堆叠累加起来。',
          type: ControlType.Boolean,
          title: '堆叠折线',
        },
        connectNulls: {
          type: ControlType.Boolean,
          title: '是否连接空数据',
        },
      },
      defaultOpen: false,
    },
  });
});
