import get from 'lodash/get';
import { LineChartProps } from './LineChart';

export const isFn = (fn: any) => typeof fn === 'function';

// 时间序列数据自动排序
export const sortTimeData = ({ xAxis = {}, data = [], xField = 'x' }: any) => {
  const type = get(xAxis, 'type', '');
  if (['time', 'timeCat'].includes(type)) {
    return data.slice().sort((a: any, b: any) => {
      if (get(a, xField) > get(b, xField)) {
        return 1;
      }
      return -1;
    });
  }
  return data;
};

// 支持 y 轴数据多选，将数据转换成 G2Plot 支持的类型
export const supportMultiYFields = (
  config: LineChartProps,
  data: Array<any>,
): Array<any> => {
  const { xField = '', yFields = new Array<any>() } = config;
  if (yFields.length > 1) {
    const splitData: Array<any> = [];
    if (xField.length > 0) {
      data.forEach(e => {
        // 由于 G2Plot 的数据支持的原因，不能直接将多个 y 坐标传入，需要转换成 seriesField 来进行点分类。
        // 比如 { id: 1, man: 10, women: 20 }，用户选取两个 y 坐标：man 和 woman 后，分割成：
        // { id: 1, y: 10, category: 'man' } 和 { id: 1, y: 20, category: 'woman' }
        yFields.forEach(y => {
          const o: any = {};
          o[xField] = e[xField];
          o.y = e[y];
          o.category = y;
          splitData.push(o);
        });
      });
      return splitData;
    } else {
      return [];
    }
  } else {
    return data;
  }
};

export const exec = (expr: string, ctx: any = {}) => {
  try {
    const vars = Object.keys(ctx || {});
    const params = vars.map(key => ctx[key]);
    /* eslint-disable no-new-func */
    // eslint-disable-next-line @typescript-eslint/no-implied-eval
    const result = new Function(...vars, `return (${expr});`)(...params);
    /* eslint-enable no-new-func */
    return result;
  } catch (ex) {
    console.info('expr=', expr, 'ctx=', ctx);
    console.error(ex);
    return expr;
  }
};

const ReplaceRegExps = [/^{([^}]+)}$/];

// 编译替换 url 中包含的模板字符串
export const compile = (inputStr: string, params: any = {}) => {
  if (typeof inputStr !== 'string') {
    return inputStr;
  }

  let str = inputStr;
  try {
    // 先统一编码再解码，避免出现非法字符解码失败
    str = decodeURIComponent(encodeURIComponent(inputStr));
    if (!str.includes('{')) {
      return str;
    }

    // 构造相关上下文信息
    const ctx = params;

    // 如果是纯编译模板字符串，直接使用 exec 得到任意类型返回值结果
    const pureRegex = ReplaceRegExps.find(regex => regex.test(str));
    if (pureRegex) {
      const match = pureRegex.exec(str);
      if (match && match.length > 1) {
        return exec(match[1], ctx);
      }
    }

    // 否则使用下面这种替换方式，只能得到字符串结果
    return str.replace(/{([^}]+)}/g, (_match, p1) => exec(p1, ctx));
  } catch (err) {
    console.error(err);
    return str;
  }
};

const AxisKeys = ['xAxis', 'yAxis'];

// 图表配置类型
type ChartOptions = any;

// 处理 formatterStr to fomatter 函数
export const compileFormatterStr = (config: ChartOptions) => {
  const { tooltip, xField, yField, yFields = [], seriesField } = config;
  const copyConfig = { ...config };

  // 当多选 yFields 时，更改 seriesFields 至默认的 category，可参考 supportMultiYFields
  if (yFields.length > 1) {
    // yFields 大于 1，
    copyConfig.seriesField = 'category';
    copyConfig.yField = 'y';
  } else if (yFields.length === 1) {
    // 只选了一个 yField，不做转换，正常处理
    copyConfig.yField = yFields[0];
  }

  // 对 legendPosition 和 legendSubPosition 做转换，legendPosition 是辅助选择图例的 options
  copyConfig.legend = {
    ...config.legend,
    position: `${config.legendPosition}${
      ['left', 'right', 'top', 'bottom'].includes(config.legendSubPosition)
        ? `-${config.legendSubPosition}`
        : ''
    }`,
  };

  // 转换线段宽度
  if (copyConfig.lineSize) {
    copyConfig.lineStyle = {
      ...copyConfig.lineStyle,
      lineWidth: copyConfig.lineSize,
    };
  }

  // x / y
  AxisKeys.forEach(key => {
    const axisLabel = config[key]?.label;
    if (axisLabel) {
      const { formatterStr, formatter, ...rest } = axisLabel;
      if (formatterStr && !formatter) {
        copyConfig[key].label = {
          ...rest,
          formatter: (value: string | number) =>
            compile(formatterStr, { value }),
        };
      }
    }
  });
  // tooltip
  if (tooltip) {
    const { formatterStr, formatter, ...rest } = tooltip;
    if (formatterStr && !formatter) {
      copyConfig.tooltip = {
        ...rest,
        formatter: (dataItem: Record<string, any>) => {
          const x = dataItem[xField];
          const y = dataItem[yField];
          const series = seriesField ? dataItem[seriesField] : yField;
          return {
            name: series,
            value: compile(formatterStr, { x, y, series }),
          };
        },
      };
    }
  }
  return copyConfig;
};
